from pathlib import Path
from string import Template
import re
from importlib import resources


class Texbrik:
    PREREQS = re.compile(r'\\prerequisite{(?P<relativepath>[/_\w]+?)}')
    BRIKINSERTS = re.compile(r'\\brikinsert{(?P<relativepath>[/_\w]+?)}')
    PKGS = re.compile(r'\\include{(\w+?)}')
    BRIKCONTENT = re.compile(r'\\begin{content}([\w\W]*?)\\end{content}')

    def __init__(self, absolute_path,
                 prerequisites, packages, content):
        if not absolute_path.is_absolute():
            raise InputError(
                str(absolute_path),
                '{path} is not an absolute path'.format(
                    path=str(absolute_path)))
        self.path          = absolute_path
        self.packages      = set(packages)
        self.content       = content
        self.prerequisites = prerequisites
        self.brikinserts   = dict()
        self.expanded      = False
        self.ignore        = set()

    def __eq__(self, other) -> bool:
        return self.path == other.path \
            if isinstance(other, Texbrik) else False

    def expand(self, ignore: set = set()):
        """recursively adds all imports from brikinsert, usepackage and brikcontent
        to content, ignoring everything mentioned in igore (except when
        explicitely inserted via brikinsert)

        Args:
            ignore (set, optional): Briks to ignore in prerequisite.
            Defaults to set().

        Raises:
            InputError: If there is something wrong with a prerequisite or
                brikinsert
        """

        self.ignore |= ignore
        if self.expanded:
            return
        try:
            self.brikinserts = dict([
                (b, brikFromDoc(
                    self._relative_pathstring_to_path(b)))
                for b in __class__.BRIKINSERTS.findall(self.content)
            ])
        except InputError as e:
            raise InputError(str(self.path),
                             'Failed to process prerequisite ' + b) from e
        self.content = __class__.BRIKINSERTS.sub(
            self._process_brikinsert_occurrence, self.content)

        s = ""
        for p in self.prerequisites:
            if p in self.ignore:
                continue
            try:
                b = brikFromDoc(
                    self._relative_pathstring_to_path(p))
            except InputError as e:
                raise InputError(str(self.path),
                                 'Failed to proces prerequisite ' + p) from e
            b.expand(ignore=self.ignore)
            self.ignore |= b.ignore
            self.ignore.add(p)
            self.packages |= b.packages
            s += b.content

        self.content = s \
            + "\n%From TeXBriK [{path}]\n".format(
                path=str(self.path)) \
            + self.content \
            + "\n%End of TeXBriK [{path}]\n".format(
                path=str(self.path))
        self.expanded = True

    def _process_brikinsert_occurrence(self, match_object):
        p = match_object[1]
        b = self.brikinserts[match_object[1]]
        b.expand(ignore=self.ignore)
        self.packages |= b.packages
        self.ignore |= b.ignore
        self.ignore.add(p)
        return b.content

    def _relative_pathstring_to_path(
            self, relative_pathstring: str) -> Path:
        p = self.path.parent.joinpath(relative_pathstring)
        b = p.with_suffix('.brik').exists()
        m = p.with_suffix('.mbrik').exists()
        if b and m:
            raise InputError(str(self.path),
                             'Both .mbrik and .brik file exists for {n}'.format(
                                 n=str(p)))
        if b:
            return p.with_suffix('.brik')
        if m:
            return p.with_suffix('.mbrik')
        raise InputError(str(self.path),
                         'No brikfile {n} exists in {p}'.format(
                             n=p.stem, p=str(p.parent)))

    def expanded_content(self) -> str:
        self.expand()
        return self.content

    #TODO default template not found - look at how data is packaged with setuptools
    def make_TeX_file(
            self,
            template: Path = False) -> str:
        """Generates LaTeX File from this TeXBriK

        Args:
            template (Path, optional): File to insert content to.
            Defaults to
                Path(__file__).resolve().parent.joinpath( 'data/default_template').

        Returns:
            str: LaTeX document's content
        """
        if not template:
            with resources.path(__package__, 'default_template.dat') as p:
                template = p

        self.expand()
        t = Template(template.read_text())
        packages = '\n'.join([
            '\\usepackage{{{}}}'.format(i) for i in self.packages])
        return t.substitute(
            packages=packages,
            content=self.content
        )

    @classmethod
    def brikFromDoc(cls, absolute_path: Path):
        """parses .brik document to create a TeXBriK object

        Args:
            absolute_path (Path): Location of the .brik document

        Raises:
            InputError: If there is a problem with the path location
            or document

        Returns:
            Texbrik: generated TeXBriK object
        """

        try:
            s = __class__.brik_document_content(absolute_path)
        except InputError as e:
            raise InputError(str(absolute_path),
                             'Failed finding Brikfile {p}'.format(
                                 p=str(absolute_path))) from e

        c = __class__.BRIKCONTENT.findall(s)
        if len(c) != 1:
            raise InputError(str(absolute_path), 'none or too many content blocks')

        return cls(
            absolute_path=absolute_path,
            prerequisites=__class__.PREREQS.findall(s),
            packages     =__class__.PKGS.findall(s),
            content      =c[0]
        )

    @staticmethod
    def brik_document_content(absolute_path: Path) -> str:
        if not absolute_path.is_absolute():
            raise InputError(
                str(absolute_path),
                '{path} is not an absuolte path'.format(path=str(absolute_path)))

        if not (absolute_path.is_file() and absolute_path.suffix.endswith('brik')):
            raise InputError(
                str(absolute_path),
                '{brik} is not a TeXBriK'.format(brik=str(absolute_path)))
        return absolute_path.read_text()

def brikFromDoc(absolute_path: Path) -> Texbrik:
    if absolute_path.suffix == '.brik':
        return Texbrik.brikFromDoc(absolute_path)
    if absolute_path.suffix == '.mbrik':
        from texbrix.mathbrik import Mathbrik
        return Mathbrik.brikFromDoc(absolute_path)
    raise InputError(str(absolute_path),
                     'Document {d} not found'.format(
                         d=str(absolute_path)))


class InputError(Exception):
    """Exception raised for errors on the input.

    Attributes:
    expression: input expression in which the error occured
    message: explanation of the error
    """

    def __init__(self, expression, message):
        self.expression = expression
        self.message = message




from . import texbrik
from . import mathbrik
import argparse
from pathlib import Path

__version__ = "0.2.9"
parser = argparse.ArgumentParser(description='create LaTeX file from brix')
parser.add_argument(
    'top_brik',
    metavar='Top Brik',
    type=str,
    help='top .brik document to work with')
parser.add_argument('--template', nargs=1)
parser.add_argument('--version', action='version', version="%(prog)s {v}".format(v=__version__))


def main():
    args = parser.parse_args()
    p = Path(args.top_brik).resolve()
    b = texbrik.brikFromDoc(p)
    if args.template:
        s = b.make_TeX_file(template=Path(args.template[0]).resolve())
    else:
        s = b.make_TeX_file()
    p.with_suffix('.tex').write_text(s)



#can not be executed directly. Execute top-level test.py instead
from texbrix.texbrik import brikFromDoc
import unittest
import sys
from pathlib import Path
sys.path.append(str(Path(__file__).resolve().parent.parent.joinpath('src')))

testdoc = Path(__file__).resolve().parent.joinpath(
    'input_files/testinput1.brik')
mbrik_testdoc = Path(__file__).resolve().parent.joinpath(
    'input_files/testinput4.mbrik')
complete_testdoc = Path(__file__).resolve().parent.joinpath(
    'input_files/testinput0.brik')


class TestTexBrik(unittest.TestCase):

    def test_packages(self):
        tb = brikFromDoc(testdoc)
        self.assertEqual(set(['amssymb']), tb.packages)

    def test_content(self):
        tb = brikFromDoc(testdoc)
        self.assertTrue(tb.content)
    
    def test_mbrik(self):
        tb = brikFromDoc(mbrik_testdoc)
        self.assertTrue(tb.content)

    def test_expand_brikinserts(self):
        tb = brikFromDoc(testdoc)
        tb.expand()
        self.assertNotEqual(tb.content.find('in2'), -1)
    
    def test_expand_mbrik(self):
        tb = brikFromDoc(complete_testdoc)
        tb.expand()
        self.assertNotEqual(tb.content.find('in2'), -1)

    def test_expand_packages(self):
        tb = brikFromDoc(testdoc)
        tb.expand()
        self.assertNotEqual([i for i in tb.packages if i != 'amssymb'], {})

    def test_expand_content(self):
        tb = brikFromDoc(testdoc)
        tb.expand()
        self.assertNotEqual(tb.content.find('in3'), -1)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import re
from pathlib import Path
from setuptools import setup, find_packages


def read(fname):
    return Path(__file__).resolve().parent.joinpath(fname).open().read()


version = re.search(
        r'^__version__\s*=\s*"(.*)"',
        open('src/texbrix/__init__.py').read(),
        re.M
        ).group(1)

setup(
    name="TeXBriX",
    version=version,
    packages=['texbrix'],
    package_dir={"": "src"},
    package_data={
        "texbrix": ["*.dat"]
    },
    entry_points={
        "console_scripts": ['texbrix=texbrix:main']
    },
    author="Leopold Fajtak",
    author_email="leopold@fajtak.at",
    description="A granular approach to LaTeX",
    license="GPLv3",
    keywords="mathematics dependencies layout filestructure",
    url="http://github.com/leopoldfajtak/texbrix",
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Topic :: Text Processing :: Markup :: LaTeX"
        ]
)



from texbrix.texbrik import Texbrik, InputError
from pathlib import Path
import re

class Mathbrik(Texbrik):

    MATHCONTENT   = re.compile(
        r'\\begin{(?P<type>\w+?)}(?P<cnt>[\w\W]*?)\\end{(?P=type)}')
    MATH_PACKAGES = set(['amsmath', 'abssymb', 'amsthm'])

    def __init__(self, absolute_path, prerequisites,
                 packages, mathcontent):

        self.mathcontent = [m.groupdict() for m in mathcontent]
        content = '\n'.join(
            [f'\\begin{{{m["type"]}}}'
             f'\n{m["cnt"]}'
             f'\n\\end{{{m["type"]}}}'
             for m in self.mathcontent])

        Texbrik.__init__(
            self=self,
            absolute_path=absolute_path,
            prerequisites=prerequisites,
            packages=packages,
            content=content)

    @classmethod
    def brikFromDoc(cls, absolute_path: Path):
        try:
            s = Texbrik.brik_document_content(absolute_path)
        except InputError as e:
            raise InputError(str(absolute_path),
                             'Failed finding Brikfile {p}'.format(
                                 p=str(absolute_path))) from e
        m = cls.MATHCONTENT.finditer(s)
        if not m:
            raise InputError(str(absolute_path),
                             'No math content in {p}'.format(
                                 p=str(absolute_path)))
        return cls(
            absolute_path=absolute_path,
            prerequisites=cls.PREREQS.findall(s),
            packages     =cls.PKGS.findall(s),
            mathcontent  =m)